; -----------------------------------------------------
; This macro accepts a quoted string as an argument,
; allocates a corresponding null-terminated string in
; the data section, and pushes the address of the
; allocated string onto the stack.
; -----------------------------------------------------
%macro PUSHSTR 1

   ; Use the primitive form of section directive to
   ; avoid changing the __SECT__ macro so it can be
   ; used to switch back to code section.
   [section .data]
      %%str: db %1,0
   __SECT__
   push  %%str

%endmacro

; -----------------------------------------------------
; This macro automatically issues an EXTERN directive
; for its first argument, but the programmer is
; responsible for adding the associated library name
; to the GoLink command line.
; -----------------------------------------------------
%macro INVOKE 1-*

  ; Declare the first argument as external so NASM
  ; will not expect it to be defined in this module.
  extern %1
   

  ; Rotate through the macro arguments from the last
  ; to the second.
  %rep %0-1
    %rotate -1
    ; If the argument is a quoted string, push a
    ; pointer to the argument. Otherwise, just
    ; push the argument.   
    %ifstr %1
      PUSHSTR %1
    %else
      push %1
    %endif
  %endrep

  ; Perform one more rotatation to restore the
  ; original order.
  %rotate -1

  ; Call the procedure.
  call %1   

%endmacro

; -----------------------------------------------------
; This macro generates code that pauses a console
; app until the user presses a key.
; -----------------------------------------------------
%macro PAUSE 0

   extern _kbhit,_getch

 %%again:	
	call  _kbhit   
	test  ax,ax
	jz	   %%again
   ; Remove the key from the buffer.
   call  _getch

%endmacro

; -----------------------------------------------------
